#include <stdexcept>
#include <stdio.h>
#include <stdlib.h>
#include "cinifile.h"

using namespace std;

CIniFile::CIniFile(const char *fn)
{
  string s;
  string::size_type pos;
  Section sect;
  vector<Section>::iterator i=sections.end();
  char str[MAX_STR_LEN];
  FILE *f;

  filename=fn;
  if((f = fopen(fn, "r"))!=NULL)
  {
    while(1)
    {
      fgets(str, MAX_STR_LEN-1, f);
      if(feof(f))
	break;
      s = str;

      //odstranime komentare
      pos = s.find_first_of(';');
      if(pos!=-1)
	s = s.substr(0, pos);

      //je to nazev sekce?
      if(s[0]=='[')
      {
	pos = s.find_first_of(']');
	if(pos!=-1)
	{
	  s = s.substr(1, pos-1);
	  sect.name=s;
	  sections.push_back(sect);
	  i=sections.end();
	  i--;	  
	}
      }
      else //je to kw=val
      {
	if(i==sections.end())//zatim neni definovana zadna sekce
	  continue;
	//orezeme bile znaky na konci radku
	pos=s.find_last_of(" \t\r\n");
	while((pos!=-1)&&(pos==s.size()-1))
	  s=s.substr(0,s.size()-1);
	pos=s.find_first_of('=');
	if(pos!=-1)//pos>=0 nefunguje
	{
	  i->keys.push_back(s);
	  i->vals.push_back(s);
	  i->keys.back()=i->keys.back().substr(0, pos);
	  i->vals.back()=i->vals.back().substr(pos+1);
	}
      }
    }

/*
    for(i=sections.begin(); i!=sections.end(); i++)
    {
      vector<string>::iterator j, k;

      for(j=i->keys.begin(), k=i->vals.begin(); j!=i->keys.end(); j++, k++)
	printf("Section %s, key %s, value %s.\n", i->name.c_str(), j->c_str(), k->c_str());
    }
*/
    fclose(f);
  }
}

CIniFile::~CIniFile()
{
  try
  {
    flush();
  }
  catch(exception &e)
  {
    printf("%s\n", e.what());
  }
}

vector<string> CIniFile::getSection(string section)
{
  vector<Section>::iterator i;

  for(i=sections.begin(); i!=sections.end(); i++)
    if(i->name==section)
      return i->keys;

  throw invalid_argument("No such section");
}

vector<string> CIniFile::getSectionValues(string section)
{
  vector<Section>::iterator i;

  for(i=sections.begin(); i!=sections.end(); i++)
    if(i->name==section)
      return i->vals;

  throw invalid_argument("No such section");
}

string CIniFile::getValue(string section, string key, string def)
{
  vector<Section>::iterator i;
  vector<string>::iterator j, k;

  for(i=sections.begin(); i!=sections.end(); i++)
    if(i->name==section)
      for(j=i->keys.begin(), k=i->vals.begin(); j!=i->keys.end(); j++, k++)
	if(*j==key)
	  return *k;

  return def;
}

int CIniFile::getInt(string section, string key, int def)
{
  char str[MAX_STR_LEN];

  snprintf(str, sizeof(str), "%d", def);
  return atoi(getValue(section, key, str).c_str());
}

void CIniFile::setValue(string section, string key, string val)
{
  vector<Section>::iterator i;
  vector<string>::iterator j, k;
  Section s;
  bool b=false;

  //najdeme sekci
  for(i=sections.begin(); i!=sections.end(); i++)
    if(i->name==section)
    {
      b=true;
      break;
    }
  //printf("%s %s\n", b?"ano":"ne", i==sections.end()?"ano":"ne");
  //kdyz neni, tak ji vytvorime
  if(!b)
  {
    s.name=section;
    sections.push_back(s);
    //nastavime i na posledni sekci
    i=sections.end();
    i--;
  }
  //printf("kuk %s\n", i==sections.begin()?"ano":"ne");
  //najdeme klic
  for(j=i->keys.begin(), k=i->vals.begin(); j!=i->keys.end(); j++, k++)
  {
    if(*j==key)
    {
      *k=val;
      return;
    }
  }
  //kdyz neni, tak ho vytvorime
  i->keys.push_back(key);
  i->vals.push_back(val);
}

void CIniFile::setInt(string section, string key, int val)
{
  char str[MAX_STR_LEN];

  snprintf(str, sizeof(str), "%d", val);
  setValue(section, key, str);
}

void CIniFile::setSectionValues(string section, vector<string>keys, vector<string>vals)
{
  vector<string>::iterator j, k;

  for(j=keys.begin(), k=vals.begin(); (j!=keys.end())&&(k!=vals.end()); j++, k++)
    setValue(section, *j, *k);
}

void CIniFile::deleteSection(string section)
{
  vector<Section>::iterator i;

  for(i=sections.begin(); i!=sections.end(); i++)
    if(i->name==section)
    {
      sections.erase(i);
      return;
    }
}

void CIniFile::deleteKey(string section, string key)
{
  vector<Section>::iterator i;
  vector<string>::iterator j, k;

  for(i=sections.begin(); i!=sections.end(); i++)
    if(i->name==section)
      for(j=i->keys.begin(), k=i->vals.begin(); j!=i->keys.end(); j++, k++)
	if(*j==key)
        {
	  i->keys.erase(j);
	  i->vals.erase(k);
	  return;
        }
}

void CIniFile::flush()
{
  vector<Section>::iterator i;
  vector<string>::iterator j, k;
  FILE *f;

//  printf("Writting to %s ...\n", filename.c_str());
  if((f=fopen(filename.c_str(), "w"))==NULL)
    throw runtime_error("Cannot write to file");
  for(i=sections.begin(); i!=sections.end(); i++)
  {
    fprintf(f, "\n[%s]\n", i->name.c_str());
    for(j=i->keys.begin(), k=i->vals.begin(); j!=i->keys.end(); j++, k++)
    {
//      printf("Section %s, key %s, value %s.\n", i->name.c_str(), j->c_str(), k->c_str());
      fprintf(f, "%s=%s\n", j->c_str(), k->c_str());
    }
  }
  fclose(f);
}
